==> LoadL-3.3.1.0-3.SLES9-X86_64.3.3.X.tar.gz: opt/ibmll/LoadL/full/samples/llres/res.c <==
/*======================================================================*/
/*                                                                      */
/* Module Name: res.c                                                   */
/*                                                                      */
/*----------------------------------------------------------------------*/
/*                                                                      */
/* Description: This sample program shows how to use the LoadLeveler    */
/* APIs to create, modify, remove and bind job steps to a reservation.  */
/* Hard coded input data are used in the sample program.                */
/* A sample for using the Data Access API to obtain data about          */
/* reservations is located in samples/lldata_access/qres.c              */
/*======================================================================*/
/* static char *sccsid = "@(#)02   1.4   src/ll/samples/llres/res.c, ll.api, ll_rcs3c2, rcs3c20538a 3/24/05 19:58:29";       */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"

char* create_reservations(void);
int change_reservation(char *);
int bind_jobs(char *reservation_id);
int remove_reservations();

main() {
	char* reservation_id = NULL;
	char command[512];
	int rc = 0;

	/* Create new reservations	*/
	reservation_id = create_reservations();
	if (!reservation_id) {
		printf("Error creating reservations.\n");
		free(reservation_id);
		exit(1);
	}

#if defined(__linux__)
	system("/opt/ibmll/LoadL/full/bin/llqres");
	sprintf(command, "/opt/ibmll/LoadL/full/bin/llqres -l -R %s", reservation_id);
#else
	system("/usr/lpp/LoadL/full/bin/llqres");
	sprintf(command, "/usr/lpp/LoadL/full/bin/llqres -l -R %s", reservation_id);
#endif

	system(command);

	/* Change the reservation	*/
	change_reservation(reservation_id);

	/* Bind job steps to the reservation */
	rc = bind_jobs(reservation_id);
	system(command);

	/* Remove existing reservations. */
	rc = remove_reservations();

#if defined(__linux__)
	system("/opt/ibmll/LoadL/full/bin/llqres");
#else
	system("/usr/lpp/LoadL/full/bin/llqres");
#endif

	free(reservation_id);
}

/* This function provides an example of using the               */
/* ll_make_reservation API to create new reservations.          */
/* In this example, input values for creating the reservations  */
/* are hard-coded. Required and optional parameters are noted.  */
/*                                                              */
/* The ID of one of the reservations is returned.               */
/*                                                              */
char* create_reservations() {
	char *errmsg = NULL;
	char *res_id = NULL;
	char *hostlist[2] = { "all", NULL };
	char *users[4];   /* Array of users that can use the reservation */
	char *groups[2];  /* Array of groups that can use the reservation */
	LL_reservation_param reservation_param; /* Structure to create reservations */
	LL_reservation_param *p_reservation_param = &reservation_param;
	LL_element *errObj = NULL; /* Error information from the API */
	int num_nodes;
	int rc =0;

	/* Initialize the structure for passing data to the API	*/
	ll_init_reservation_param(LL_API_VERSION, &errObj, &p_reservation_param);

	/* Populate the reservation_param structure */

	/* Array of users allowed to use the reservation is optional.	*/
	users[0] = "dave";
	users[1] = "carol";
	users[2] = "rich";
	users[3] = NULL;
	reservation_param.users = users;

	/* Array of LL groups allowed to use the reservation is optional.*/
	groups[0] = "DevGroup";
	groups[1] = NULL;
	reservation_param.groups = groups;

	/* Optionally, provide the mode.                       */
	/* In this example REMOVE_ON_IDLE mode is not enabled. */
	/*             SHARED mode is enabled.                 */
	reservation_param.mode = RESERVATION_SHARED;

	/* LoadLeveler group that will own the reservation is optional.	*/
	reservation_param.group = "AdminGroup";

	/* The reservation will be scheduled with a duration of */
	/* 30 minutes. Duration is required.			              */
	reservation_param.duration = 30;

	/* Start time of the reservation must be in the format: */
	/* [mm/dd[/[cc]yy] ]HH:MM. Start time is required.      */

	/* Using short-hand notation to schedule the */
	/* reservation for 8pm today.               */
	reservation_param.start_time = "20:00";

	/* The method and data that will be used to reserve nodes */
	/* must be specified.                                     */

	/* Reserve 1 node */
	reservation_param.data_type = RESERVATION_BY_NODE;
	num_nodes = 1;
	reservation_param.data = &num_nodes;

	/* Create the reservation */
	printf("Create a shared reservation with 1 node at 8pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);

	/* On error, print error message from errObj */
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		/* On success */
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		/* Free the unused memory allocations */
		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving all available nodes */
	reservation_param.start_time = "21:00";
	reservation_param.mode = RESERVATION_REMOVE_ON_IDLE;
	reservation_param.data_type = RESERVATION_BY_HOSTLIST;
	reservation_param.data = (void*) hostlist;

	printf("Create a reservation with all nodes and REMOVE_ON_IDLE option at 9pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving nodes for a job step */
	reservation_param.start_time = "22:00";
	reservation_param.mode = RESERVATION_REMOVE_ON_IDLE & RESERVATION_SHARED;
	reservation_param.data_type = RESERVATION_BY_JOBSTEP;
	reservation_param.data = (void*) "c94n04.1.0";

	printf("Create a reservation with both SHARED and REMOVE_ON_IDLE options to run job step c94n04.1.0 at 10pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving nodes for the first job step of a job command file */
	reservation_param.start_time = "23:00";
	reservation_param.mode = RESERVATION_DEFAULT_MODE;
	reservation_param.data_type = RESERVATION_BY_JCF;
	reservation_param.data = (void*) "A.cmd";

	printf("Create a reservation using job command file A.cmd at 11pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}

	return res_id;
}


/* This function provides an example of using the           */
/* ll_change_reservation API to change the reservation.     */
/* In this example, values for changing the reservation are */
/* hard-coded. Reservation ID is the only required input  */
/* to the ll_change_reservation API.                        */
/*                                                          */
/* The return code from ll_change_reservation is returned.  */
/*                                                          */
int change_reservation(char *reservation_id) {
#define NUM_CHANGES 4
	char *users[3] = {"iris", "nate", NULL}; /* Array of users that can use the reservation */
	char *groups[2] = {"DevGroup", NULL};    /* Array of groups that can not use the reservation any longer */
	LL_reservation_change_param change_struct[NUM_CHANGES];  /* for change parameters */
	LL_reservation_change_param *p_change_struct[NUM_CHANGES+1];
	int add_start_time;
	int add_duration;

	int rc = 0;
	int i;
	LL_element *errObj = NULL;   /* Error information from API */

	/* initialize the array of pointers, must be NULL terminated */
	for (i=0; i<NUM_CHANGES; i++) {
		p_change_struct[i] = &change_struct[i];
	}
	p_change_struct[i] = NULL;

	/* Populate the change_struct structure */

	/* Change the reservation to begin 10 minutes earlier	*/
	change_struct[0].type = RESERVATION_ADD_START_TIME;
	add_start_time = -10;
	change_struct[0].data = &add_start_time;

	/* Change the reservation to last 10 minutes longer.		*/
	change_struct[1].type = RESERVATION_ADD_DURATION;
	add_duration = 10;
	change_struct[1].data = &add_duration;

	/* Add 2 users to the current list of users who are	*/
	/* allowed to use the reservation.			*/
	change_struct[2].type = RESERVATION_ADD_USERS;
	change_struct[2].data = (void*)users;

	/* Change the list of groups who are allowed to use	*/
	/* the reservation.                                	*/
	change_struct[3].type = RESERVATION_DEL_GROUPS;
	change_struct[3].data = (void*)groups;

	/* Invoke ll_change_reservation to initiate the change	*/
	printf("\nChange reservation %s (start 10 min earlier, last 10 min longer, add 2 users, delete 1 group):\n", reservation_id);
	rc = ll_change_reservation(LL_API_VERSION, &errObj, &reservation_id, p_change_struct);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_change_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Reservation %s has been changed.\n\n", reservation_id);
	}
	return(rc);
}


/* This function provides an example of using the ll_bind       */
/* API to bind jobs in an idle-like state to the reservation    */
/* that was previously created.                                 */
/* In this example, the job names are hard-coded.               */
/* When binding a job or job step to a reservation, the         */
/* reservation ID and job/step name(s) are the only required    */
/* input to the ll_bind API. To unbind a job or job step, using */
/* the -r flag, only the job or job step name are required.     */
/*                                                              */
/* The return code from ll_bind is returned.                    */
/*                                                              */
int bind_jobs(char *reservation_id) {
	LL_bind_param   bind_param;	   /* struct for bind data        */
	LL_bind_param   *p_bind_param = &bind_param;
	char *joblist[4];              /* Array of jobs/steps to bind */
	int rc = 0;
	LL_element *errObj = NULL;     /* Error info from API         */

	/* Bind all steps of jobs c94n04.1 and c94n04.2	*/
	/* and only job step c94n04.3.0 to the reservation.	*/
	joblist[0] = "c94n04.1";
	joblist[1] = "c94n04.2";
	joblist[2] = "c94n04.3.0";
	joblist[3] = NULL;

	bind_param.jobsteplist = joblist;

	/* Identify the reservation to which jobs/steps	will be bound	*/
	bind_param.ID = reservation_id;

	/* Jobs are being bound (not unbound) to the reservation*/
	bind_param.unbind = FALSE;

	/* Invoke ll_bind to initiate the bind/unbind process	*/
	printf("Bind job steps to reservation %s:\n", reservation_id);
	rc = ll_bind(LL_API_VERSION, &errObj, &p_bind_param);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_bind() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Request has been sent to LoadLeveler.\n\n");
	}
	return(rc);
}


/* This function provides an example of using the               */
/* ll_remove_reservation API to remove "all" reservations.      */
/*                                                              */
/* The return code from ll_remove_reservation is returned.      */
/*                                                              */
int remove_reservations() {
	char *IDs[2] = {"all", NULL};  /* reservation list */
	int rc = 0;
	LL_element *errObj = NULL;  /* Error info from API          */

	/* Remove all reservations */
	printf("\nRemove all reservations:\n");
	rc = ll_remove_reservation(LL_API_VERSION, &errObj, IDs, NULL, NULL, NULL);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_remove_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Request has been sent to LoadLeveler.\n\n");
	}
	return(rc);
}

==> LoadL-3.4.0.4-0.SLES9-X86_64.3.4.X.tar.gz: opt/ibmll/LoadL/full/samples/llres/res.c <==
/*======================================================================*/
/*                                                                      */
/* Module Name: res.c                                                   */
/*                                                                      */
/*----------------------------------------------------------------------*/
/*                                                                      */
/* Description: This sample program shows how to use the LoadLeveler    */
/* APIs to create, modify, remove and bind job steps to a reservation.  */
/* Hard coded input data are used in the sample program.                */
/* A sample for using the Data Access API to obtain data about          */
/* reservations is located in samples/lldata_access/qres.c              */
/*======================================================================*/
/* static char *sccsid = "@(#)02   1.6   src/ll/samples/llres/res.c, ll.api, ll_rven, rven0635b 8/1/06 17:24:38";       */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"

char* create_reservations(void);
int change_reservation(char *);
int bind_jobs(char *reservation_id);
int remove_reservations();

main() {
	char* reservation_id = NULL;
	char command[512];
	int rc = 0;

	/* Create new reservations	*/
	reservation_id = create_reservations();
	if (!reservation_id) {
		printf("Error creating reservations.\n");
		free(reservation_id);
		exit(1);
	}

#if defined(__linux__)
	system("/opt/ibmll/LoadL/full/bin/llqres");
	sprintf(command, "/opt/ibmll/LoadL/full/bin/llqres -l -R %s", reservation_id);
#else
	system("/usr/lpp/LoadL/full/bin/llqres");
	sprintf(command, "/usr/lpp/LoadL/full/bin/llqres -l -R %s", reservation_id);
#endif

	system(command);

	/* Change the reservation	*/
	change_reservation(reservation_id);

	/* Bind job steps to the reservation */
	rc = bind_jobs(reservation_id);
	system(command);

	/* Remove existing reservations. */
	rc = remove_reservations();

#if defined(__linux__)
	system("/opt/ibmll/LoadL/full/bin/llqres");
#else
	system("/usr/lpp/LoadL/full/bin/llqres");
#endif

	free(reservation_id);
}

/* This function provides an example of using the               */
/* ll_make_reservation API to create new reservations.          */
/* In this example, input values for creating the reservations  */
/* are hard-coded. Required and optional parameters are noted.  */
/*                                                              */
/* The ID of one of the reservations is returned.               */
/*                                                              */
char* create_reservations() {
	char *errmsg = NULL;
	char *res_id = NULL;
	char *hostlist[2] = { "free", NULL };
	char *users[4];   /* Array of users that can use the reservation */
	char *groups[2];  /* Array of groups that can use the reservation */
	LL_reservation_param reservation_param; /* Structure to create reservations */
	LL_reservation_param *p_reservation_param = &reservation_param;
	LL_element *errObj = NULL; /* Error information from the API */
	int num_nodes;
	int rc =0;

	/* Initialize the structure for passing data to the API	*/
	ll_init_reservation_param(LL_API_VERSION, &errObj, &p_reservation_param);

	/* Populate the reservation_param structure */

	/* Array of users allowed to use the reservation is optional.	*/
	users[0] = "dave";
	users[1] = "carol";
	users[2] = "rich";
	users[3] = NULL;
	reservation_param.users = users;

	/* Array of LL groups allowed to use the reservation is optional.*/
	groups[0] = "DevGroup";
	groups[1] = NULL;
	reservation_param.groups = groups;

	/* Optionally, provide the mode.                       */
	/* In this example REMOVE_ON_IDLE mode is not enabled. */
	/*             SHARED mode is enabled.                 */
	reservation_param.mode = RESERVATION_SHARED;

	/* LoadLeveler group that will own the reservation is optional.	*/
	reservation_param.group = "AdminGroup";

	/* The reservation will be scheduled with a duration of */
	/* 30 minutes. Duration is required.			              */
	reservation_param.duration = 30;

	/* Start time of the reservation must be in the format: */
	/* [mm/dd[/[cc]yy] ]HH:MM. Start time is required.      */

	/* Using short-hand notation to schedule the */
	/* reservation for 8pm today.               */
	reservation_param.start_time = "20:00";

	/* The method and data that will be used to reserve nodes */
	/* must be specified.                                     */

	/* Reserve 1 node */
	reservation_param.data_type = RESERVATION_BY_NODE;
	num_nodes = 1;
	reservation_param.data = &num_nodes;

	/* Create the reservation */
	printf("Create a shared reservation with 1 node at 8pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);

	/* On error, print error message from errObj */
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		/* On success */
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		/* Free the unused memory allocations */
		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving all free nodes */
	reservation_param.start_time = "21:00";
	reservation_param.mode = RESERVATION_REMOVE_ON_IDLE;
	reservation_param.data_type = RESERVATION_BY_HOSTLIST;
	reservation_param.data = (void*) hostlist;

	printf("Create a reservation with all nodes and REMOVE_ON_IDLE option at 9pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving nodes for a job step */
	reservation_param.start_time = "22:00";
	reservation_param.mode = RESERVATION_REMOVE_ON_IDLE & RESERVATION_SHARED;
	reservation_param.data_type = RESERVATION_BY_JOBSTEP;
	reservation_param.data = (void*) "c94n04.1.0";

	printf("Create a reservation with both SHARED and REMOVE_ON_IDLE options to run job step c94n04.1.0 at 10pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Create another reservation reserving nodes for the first job step of a job command file */
	reservation_param.start_time = "23:00";
	reservation_param.mode = RESERVATION_DEFAULT_MODE;
	reservation_param.data_type = RESERVATION_BY_JCF;
	reservation_param.data = (void*) "A.cmd";

	printf("Create a reservation using job command file A.cmd at 11pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		if (res_id) free(res_id);
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}


	/* Reserve 32 BG c-nodes */
	reservation_param.data_type = RESERVATION_BY_BG_CNODE;
	num_nodes = 32;
	reservation_param.data = &num_nodes;

	/* Create the reservation */
	printf("Create a shared reservation with 32 BG c-nodes at 8pm today:\n");
	rc = ll_make_reservation(LL_API_VERSION, &errObj, &p_reservation_param);

	/* On error, print error message from errObj */
	if (rc) {
		errmsg = ll_error(&errObj, 0);
		printf("ll_make_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		/* On success */
		res_id = strdup(*p_reservation_param->ID);
		printf("Reservation %s has been created.\n\n", res_id);

		/* Free the unused memory allocations */
		free(*p_reservation_param->ID);
		free(p_reservation_param->ID);
		p_reservation_param->ID = NULL;
	}

	return res_id;
}


/* This function provides an example of using the           */
/* ll_change_reservation API to change the reservation.     */
/* In this example, values for changing the reservation are */
/* hard-coded. Reservation ID is the only required input  */
/* to the ll_change_reservation API.                        */
/*                                                          */
/* The return code from ll_change_reservation is returned.  */
/*                                                          */
int change_reservation(char *reservation_id) {
#define NUM_CHANGES 4
	char *users[3] = {"iris", "nate", NULL}; /* Array of users that can use the reservation */
	char *groups[2] = {"DevGroup", NULL};    /* Array of groups that can not use the reservation any longer */
	LL_reservation_change_param change_struct[NUM_CHANGES];  /* for change parameters */
	LL_reservation_change_param *p_change_struct[NUM_CHANGES+1];
	int add_start_time;
	int add_duration;

	int rc = 0;
	int i;
	LL_element *errObj = NULL;   /* Error information from API */

	/* initialize the array of pointers, must be NULL terminated */
	for (i=0; i<NUM_CHANGES; i++) {
		p_change_struct[i] = &change_struct[i];
	}
	p_change_struct[i] = NULL;

	/* Populate the change_struct structure */

	/* Change the reservation to begin 10 minutes earlier	*/
	change_struct[0].type = RESERVATION_ADD_START_TIME;
	add_start_time = -10;
	change_struct[0].data = &add_start_time;

	/* Change the reservation to last 10 minutes longer.		*/
	change_struct[1].type = RESERVATION_ADD_DURATION;
	add_duration = 10;
	change_struct[1].data = &add_duration;

	/* Add 2 users to the current list of users who are	*/
	/* allowed to use the reservation.			*/
	change_struct[2].type = RESERVATION_ADD_USERS;
	change_struct[2].data = (void*)users;

	/* Change the list of groups who are allowed to use	*/
	/* the reservation.                                	*/
	change_struct[3].type = RESERVATION_DEL_GROUPS;
	change_struct[3].data = (void*)groups;

	/* Invoke ll_change_reservation to initiate the change	*/
	printf("\nChange reservation %s (start 10 min earlier, last 10 min longer, add 2 users, delete 1 group):\n", reservation_id);
	rc = ll_change_reservation(LL_API_VERSION, &errObj, &reservation_id, p_change_struct);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_change_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Reservation %s has been changed.\n\n", reservation_id);
	}
	return(rc);
}


/* This function provides an example of using the ll_bind       */
/* API to bind jobs in an idle-like state to the reservation    */
/* that was previously created.                                 */
/* In this example, the job names are hard-coded.               */
/* When binding a job or job step to a reservation, the         */
/* reservation ID and job/step name(s) are the only required    */
/* input to the ll_bind API. To unbind a job or job step, using */
/* the -r flag, only the job or job step name are required.     */
/*                                                              */
/* The return code from ll_bind is returned.                    */
/*                                                              */
int bind_jobs(char *reservation_id) {
	LL_bind_param   bind_param;	   /* struct for bind data        */
	LL_bind_param   *p_bind_param = &bind_param;
	char *joblist[4];              /* Array of jobs/steps to bind */
	int rc = 0;
	LL_element *errObj = NULL;     /* Error info from API         */

	/* Bind all steps of jobs c94n04.1 and c94n04.2	*/
	/* and only job step c94n04.3.0 to the reservation.	*/
	joblist[0] = "c94n04.1";
	joblist[1] = "c94n04.2";
	joblist[2] = "c94n04.3.0";
	joblist[3] = NULL;

	bind_param.jobsteplist = joblist;

	/* Identify the reservation to which jobs/steps	will be bound	*/
	bind_param.ID = reservation_id;

	/* Jobs are being bound (not unbound) to the reservation*/
	bind_param.unbind = FALSE;

	/* Invoke ll_bind to initiate the bind/unbind process	*/
	printf("Bind job steps to reservation %s:\n", reservation_id);
	rc = ll_bind(LL_API_VERSION, &errObj, &p_bind_param);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_bind() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Request has been sent to LoadLeveler.\n\n");
	}
	return(rc);
}


/* This function provides an example of using the               */
/* ll_remove_reservation API to remove "all" reservations.      */
/*                                                              */
/* The return code from ll_remove_reservation is returned.      */
/*                                                              */
int remove_reservations() {
	char *IDs[2] = {"all", NULL};  /* reservation list */
	int rc = 0;
	LL_element *errObj = NULL;  /* Error info from API          */

	/* Remove all reservations */
	printf("\nRemove all reservations:\n");
	rc = ll_remove_reservation(LL_API_VERSION, &errObj, IDs, NULL, NULL, NULL, NULL);

	/* On error, print error message from errObj */
	if (rc) {
		char *errmsg = ll_error(&errObj, 0);
		printf("ll_remove_reservation() return code: %d\n%s\n", rc, errmsg);
		free(errmsg);
	} else {
		printf("Request has been sent to LoadLeveler.\n\n");
	}
	return(rc);
}
